@@ -21,7 +21,6 @@ import (
 	"context"
 	"fmt"
 	"github.com/IrineSistiana/mosdns/v2/dispatcher/handler"
-	"github.com/miekg/dns"
 	"go.uber.org/zap"
 	"time"
 )
@@ -33,16 +32,16 @@ type ParallelNode struct {
 	logger *zap.Logger // not nil
 }
 
+const (
+	defaultParallelTimeout = time.Second * 5
+)
+
 type ParallelConfig struct {
 	Parallel []interface{} `yaml:"parallel"`
 	Timeout  uint          `yaml:"timeout"`
 }
 
 func ParseParallelNode(c *ParallelConfig, logger *zap.Logger) (*ParallelNode, error) {
-	if len(c.Parallel) < 2 {
-		return nil, fmt.Errorf("parallel needs at least 2 cmd sequences, but got %d", len(c.Parallel))
-	}
-
 	ps := make([]handler.ExecutableChainNode, 0, len(c.Parallel))
 	for i, subSequence := range c.Parallel {
 		es, err := ParseExecutableNode(subSequence, logger)
@@ -63,10 +62,9 @@ func ParseParallelNode(c *ParallelConfig, logger *zap.Logger) (*ParallelNode, er
 }
 
 type parallelECSResult struct {
-	r      *dns.Msg
-	status handler.ContextStatus
-	err    error
-	from   int
+	qCtx *handler.Context
+	err  error
+	from int
 }
 
 func (p *ParallelNode) Exec(ctx context.Context, qCtx *handler.Context, next handler.ExecutableChainNode) error {
@@ -77,6 +75,9 @@ func (p *ParallelNode) Exec(ctx context.Context, qCtx *handler.Context, next han
 }
 
 func (p *ParallelNode) exec(ctx context.Context, qCtx *handler.Context) error {
+	if len(p.s) == 0 {
+		return nil
+	}
 
 	t := len(p.s)
 	c := make(chan *parallelECSResult, len(p.s)) // use buf chan to avoid blocking.
@@ -91,7 +92,10 @@ func (p *ParallelNode) exec(ctx context.Context, qCtx *handler.Context) error {
 		if p.timeout > 0 {
 			pCtx, cancel = context.WithTimeout(context.Background(), p.timeout)
 		} else {
-			pCtx, cancel = context.WithCancel(ctx)
+			if ddl, ok := ctx.Deadline(); ok {
+				pCtx, cancel = context.WithDeadline(ctx, ddl)
+			}
+			pCtx, cancel = context.WithTimeout(ctx, defaultParallelTimeout)
 		}
 
 		go func() {
@@ -99,10 +103,9 @@ func (p *ParallelNode) exec(ctx context.Context, qCtx *handler.Context) error {
 
 			err := handler.ExecChainNode(pCtx, qCtxCopy, n)
 			c <- &parallelECSResult{
-				r:      qCtxCopy.R(),
-				status: qCtxCopy.Status(),
-				err:    err,
-				from:   i,
+				qCtx: qCtxCopy,
+				err:  err,
+				from: i,
 			}
 		}()
 	}
