@@ -19,6 +19,7 @@ package hosts
 
 import (
 	"fmt"
+	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/dnsutils"
 	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/matcher/domain"
 	"github.com/miekg/dns"
 	"net"
@@ -89,6 +90,7 @@ func (h *Hosts) LookupMsg(m *dns.Msg) *dns.Msg {
 
 	r := new(dns.Msg)
 	r.SetReply(m)
+	r.RecursionAvailable = true
 	switch {
 	case typ == dns.TypeA && len(ipv4) > 0:
 		for _, ip := range ipv4 {
@@ -121,6 +123,11 @@ func (h *Hosts) LookupMsg(m *dns.Msg) *dns.Msg {
 			r.Answer = append(r.Answer, rr)
 		}
 	}
+
+	// Append fake SOA record for empty reply.
+	if len(r.Answer) == 0 {
+		r.Ns = []dns.RR{dnsutils.FakeSOA(fqdn)}
+	}
 	return r
 }
 
